examplegen

package
v1.9.0-alpha.6 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 28, 2026 License: MIT Imports: 4 Imported by: 0

Documentation

Overview

Package examplegen derives deterministic generators for stable codegen scopes.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func ForScope

func ForScope(generator *expr.ExampleGenerator, parts ...string) *expr.ExampleGenerator

ForScope derives an independent generator for parts when generator uses Loom's seeded faker. Custom generators retain their caller-defined behavior.

Types

This section is empty.

Jump to