logrusbridge

package
v1.0.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 22, 2026 License: MIT Imports: 3 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func New

func New(cfg Config) (*logrus.Logger, error)

New returns a logrus logger that mirrors entries into OpenTelemetry logs using the official otellogrus bridge.

Types

type Config

type Config struct {
	// ServiceName is the instrumentation name attached to bridged logs.
	ServiceName string
	// LoggerProvider overrides the OpenTelemetry logger provider used by the
	// otellogrus bridge.
	LoggerProvider log.LoggerProvider
}

Config configures a logrus logger bridged into OpenTelemetry logs.

Jump to